/*
* Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation, 
* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/
#include <qapplication.h> 
#include <qclipboard.h> 
#include <qlistbox.h>
#include <qlayout.h>
#include <qwidget.h>
#include <qlineedit.h>
#include <qfile.h>
#include <qdir.h>
#include <qtextstream.h>
#include <qcombobox.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qpushbutton.h>
#include <qtooltip.h>

#include "dfstatusbar.h"
#include "listboxmanager.h"
#include "centralwidget.h"
#include "mybrowser.h"
#include "Dico.h"
#include "sentenceline.h"
#include "searchinterface.h"
#include "preferences.h"

typedef vector<QString>::iterator strIt;
typedef vector<HistoryEvent *>::iterator heIt;

CentralWidget::CentralWidget( MainWindow* parent, DFStatusBar* statusbar, const char *name )
: QSplitter( parent, name ),m_mainwindow(parent)
{

	m_bBuilt = false;
	m_statusbar=statusbar;
	
	currentDisplayed = NULL;
	m_currentDico=NULL;
	m_currentPage=NULL;
	m_historyCurrent=0;
	
	setHistoryWriteable(false);
	backpix = new QPixmap(qApp->applicationDirPath()+"/../images/backward.png");
	forwpix=new QPixmap(qApp->applicationDirPath()+"/../images/forward.png");
	backtranspix=new QPixmap(qApp->applicationDirPath()+"/../images/backtrans.png");	
	forwtranspix=new QPixmap(qApp->applicationDirPath()+"/../images/forwtrans.png");
	
	//Strucutre du composant : deux colonnes, left et right
	//left contient 2 places
	//right en contient 1
	//soit 3 cases :
	
	//---------------
	//|    |         |
	//|    |         |
	//| 1  |    2    |
	//|    |         |
	//|    |         |
	//----------------
	
	setFrameStyle( QFrame::Box | QFrame::Sunken );
	
	//m_left est le composant 1, un composant  onglets
	m_left = new QTabWidget(this);
	
	//right est le composant 2, il va contenir l'onglet  dicos et la vue HTML
	right = new QVBox(this);
	
	setResizeMode(m_left,QSplitter::Stretch);
	setResizeMode(right,QSplitter::Stretch );
	
	QValueList<int> sizes;
	sizes.append(25);	//25% occups
	sizes.append(75);	//75% occups
	setSizes ( sizes );
	
	
	//Remplissons la colonne 2
	QHBox* upperright = new QHBox(right);
	m_browser = new MyBrowser(this,right);		//voici la vue HTML
	QVBox* bottomright = new QVBox(right);
	bottomright->setMargin(2);
	bottomright->setSpacing(1);

	m_sentenceline = new SentenceLine(this,bottomright);
	m_tengwarview = new QTextBrowser(bottomright);
	m_tengwarview->setFixedHeight(50);
	m_tengwarview->setLineWidth(1);
	m_tengwarview->setMidLineWidth(0);
	bottomright->setFixedHeight(bottomright->sizeHint().height());
	m_tengwarview->setFrameStyle( QFrame::Box | QFrame::Sunken );
//	m_sentenceline->setFrameStyle( QFrame::Box | QFrame::Sunken );
	
	backbutton = new QPushButton(upperright);
	forwardbutton = new QPushButton(upperright);
	connect(	backbutton,		SIGNAL(clicked ()),this,SLOT(backward()));
	connect(	forwardbutton,	SIGNAL(clicked ()),this,SLOT(forward()));
	backbutton->setFocusPolicy(QWidget::NoFocus);
	forwardbutton->setFocusPolicy(QWidget::NoFocus);
	
	backbutton->setFixedSize(42,26); 
	forwardbutton->setFixedSize(42,26); 
	backbutton->setPixmap(*backpix);
	forwardbutton->setPixmap(*forwpix);
	backbutton->setEnabled( false );
	forwardbutton->setEnabled( false );
	
	m_dicocombo = new QComboBox(FALSE,upperright);	//voici l'onglet  dicos
	
	
	//On va rendre notre vue HTML jolie avec une tite image
	QBrush paper;
    paper.setPixmap( QPixmap( qApp->applicationDirPath()+"/../images/paper.png" ) );
    m_browser->setPaper( paper );
	m_tengwarview->setPaper( paper );
	//m_tengwarview->setMargin(3);
	QFont fteng("Tengwar Sindarin",15);

	QString unifontname = DFConfig::instance()->getString( "fonts", "unicode", "Lucida Sans Unicode" );
	QFont unifont( unifontname, 8 );
	m_sentenceline->setFont( unifont );
	m_tengwarview->setFont(fteng);
	m_tengwarview->setHScrollBarMode ( QScrollView::AlwaysOff );
	
	//et un peu de mises en forme sur les marges et les espacements
	right->setSpacing(1);
	
	//on veut pas de focus sur tous ces composants, sinon pour le tab
	//a va gner
	m_dicocombo->setFocusPolicy(QWidget::NoFocus);
	m_left->setFocusPolicy(QWidget::NoFocus);
	m_tengwarview->setFocusPolicy(QWidget::NoFocus);
	m_browser->setFocusPolicy(QWidget::NoFocus);
	m_sentenceline->setFocusPolicy(QWidget::ClickFocus);
	

	//hop, y'a plus qu' connecter les signaux
	//le premier, c'est pour chager de dico, quand on le slectionne
	//avec la combobox en haut  droite
	connect(m_dicocombo,SIGNAL( activated(int) ),SLOT( dicoClickedSelected(int)) );
	//le second, c'est pour changer de langue quand on clique sur un onglet
	//sur le composant de la colonne1
	connect(m_left,SIGNAL(currentChanged ( QWidget *)),this,SLOT(pageSelected( QWidget *)));
	
    QToolTip::add( backbutton, "Go back history" );
	QToolTip::add( forwardbutton, "Go forward history" );
	QToolTip::add( m_dicocombo, "Current dictionary" );
	
	loadModes();
	
	//Bon maintenant qu'on a l'interface, chargeons les dicos.
	loadDicos();
	
	//Bon bin, si on a au moins un dico, on le montre.
	if(m_dicocombo->count()>0)
	{
		HistoryEvent* he = new HistoryEvent();
		he->dicoindex = 0;
		he->page=1;
		he->numberofentry=0;
		
		m_history.push_back(he);
		
		showDico(0);
		
	}

}

void CentralWidget::loadModes()
{
	m_modenames.push_back("None");
	
	//On rcupre les fichiers qu'il y a dans le rpertoire "modes"
	QDir d(qApp->applicationDirPath()+"/../modes");
	d.setFilter( QDir::Files  );
	d.setSorting( QDir::Name); //par ordre alphab
	const QFileInfoList *list = d.entryInfoList();
	if(!list) return;
	QFileInfoListIterator it( *list );      //pointeur pour parcourir la liste
	QFileInfo *fi;     
	
	it.toFirst();	
	
    while ( (fi=it.current()) ) 
	{
		//alors, on a trouv un point mod ?
		if(fi->extension( FALSE )=="mod")
		{	
			m_modenames.push_back(fi->baseName());
			
			CTranscription* transcriptor=new CTranscription();
			transcriptor->SetDigits(false);
			transcriptor->SetLSD(false);
			transcriptor->SetSmart(false);
			transcriptor->SetDecimal(false);
			transcriptor->SetSeparateNums(true);
			transcriptor->SetAlert(false);
			transcriptor->LoadMode( QString(qApp->applicationDirPath()+"/../modes/"+fi->baseName()+".mod").latin1());

			m_modes.push_back(transcriptor);
        }
		
		++it;
	}
	
}


void CentralWidget::loadDicos()
{
	
	//C'est parti
	
	//On rcupre les fichiers qu'il y a dans le rpertoire dicos
	QDir d(qApp->applicationDirPath()+"/../dicos");
	d.setFilter( QDir::Files  );
	d.setSorting( QDir::Name);
	const QFileInfoList *list = d.entryInfoList();
	if(!list) return;
	QFileInfoListIterator it( *list );      //pointeur pour parcourir la liste
	QFileInfo *fi;     
	
	it.toFirst();	
	
    while ( (fi=it.current()) ) 
	{
		//alors, on a trouv un point ini ?
		if(fi->extension( FALSE )=="ini")
		{	
			loadDico(fi->baseName());
        }
		
		++it;
	}
}


void CentralWidget::loadDico(QString diconame)
{
	
	QString dicoheader;
	int numberofpages;
	
	int i;
	
	QFile fini(QString(qApp->applicationDirPath()+"/../dicos/"+diconame+".ini"));
	
	fini.open( IO_ReadOnly );
	
	//le .ini existe, voyons voir ce qu'il a dans le ventre
	QTextStream tini(&fini);
	
	dicoheader		= tini.readLine();
	numberofpages	= (tini.readLine()).toInt();
	
	if(numberofpages<=0||numberofpages>999) return;
	
	for(i=1;i<=numberofpages;i++)
	{
		QString extension(QString("%1").arg(i));
		extension=extension.rightJustify(3,'0');
		
//		const char* extensionlat = extension.latin1();
		
		QFile currentpagefile(QString(qApp->applicationDirPath()+"/../dicos/"+diconame+"."+extension));
		
		if(!currentpagefile.open( IO_ReadOnly )) return;
		
		currentpagefile.close();
	}
	
	//Hop. Premire ligne = ce qu'on va mettre dans la combobox.
	if (m_dicocombo->count() % 2)
	{
		m_dicocombo->insertItem(QPixmap(qApp->applicationDirPath()+"/../images/smbkblue.png"),dicoheader);
	}
	else
	{
		m_dicocombo->insertItem(QPixmap(qApp->applicationDirPath()+"/../images/smbkred.png"),dicoheader);
	}
	//Retenons le nom gnrique du fichier dans un vecteur
	m_diconames.push_back(diconame);
	//crons le dico
	Dico* newdico = new Dico(diconame);
	//bufferisons le dico
	for(i=1;i<=numberofpages;i++)
	{
		QString extension(QString("%1").arg(i));
		extension=extension.rightJustify(3,'0');
		QFile currentpagefile(QString(qApp->applicationDirPath()+"/../dicos/"+diconame+"."+extension));
		
		Page* newpage=new Page(this,newdico,m_left);
		
		//Alors, maintenant lisons le fichier
		currentpagefile.open(IO_ReadOnly);
		QTextStream tst(&currentpagefile);		//un stream pour lire le fichier.
		QString sbuffer;						//un buffer
		QString htmlentry("");					//l'entre html en cours
		QString entryname;						//le nom de l'entre en cours
		
		//mangeons le dbut du fichier, qui ne nous intresse pas
		sbuffer = tst.readLine();
		while(!sbuffer.startsWith("<!-- @@@@@@@ ")&&!tst.atEnd()) sbuffer = tst.readLine();	
		//A y'est fini.
		
		//Allons-y maintenant...
		while(!tst.atEnd())
		{
			//on rcupre le nom de l'entre, en faisant sauter 
			// "<!-- @@@@@@@ " et " -->"
			// soit 13 + 4 caractres = 17
			entryname = sbuffer.mid(13,sbuffer.length()-17);
			
			htmlentry="";
			sbuffer="";
			
			//mangeons l'html et concatnons le, jusqu' la prochaine
			//entre ou la fin du fichier
			while(!sbuffer.startsWith("<!-- @@@@@@@ ")&&!tst.atEnd())
			{
				htmlentry+=sbuffer;
				sbuffer = tst.readLine();
			}
			
			//si c'tait la dernire ligne, penser  concatner quand mme celle ci
			//au contenu html de l'entre en cours
			if(tst.atEnd()) htmlentry+=sbuffer;
			
			//crons la nouvelle entre, l'html tant mis en lucida sans unicore
			Entry* newone = new Entry(newpage->manager(),clean(entryname),htmlentry);
			
			//et comme c'est le .ten, on bufferise a dans la page1
			newpage->manager()->insertewoconneries(newone);
		}
		
		currentpagefile.close();
		
		newpage->loadZero();
		newpage->setProperties(tini.readLine(),diconame,i,tini.readLine(),m_modenames);
		
		newdico->addPage(newpage);
	}
	
	newdico->connectMyPages();
	newdico->connectMyPagesToTheTabWidget(m_left);
	m_dicos.push_back(newdico);
	
	fini.close();
}

void CentralWidget::dicoClickedSelected(int index)
{
	
	int i;
	int currentsize;
	//Appele quand on clique sur la combo en haut  droite
	
	if(m_currentDico)
	{
		currentsize=m_currentDico->size();
		for(i=0;i<currentsize;i++) m_left->removePage(m_currentDico->page(i));
	}
	m_currentDico=m_dicos[index];
	//on retient le nom du fichier html qu'il faudra montrer quand on clique sur help
	m_hlpname= m_diconames[index]+".html";
	
	//on montre les nouvelles pages du dico..euh on ne se moque pas :p
	currentsize=m_currentDico->size();
	for(i=0;i<currentsize;i++)
	{
		Page* curpage=m_currentDico->page(i);
		m_left->addTab(curpage,curpage->title());
		
		m_left->showPage(curpage);
		
	}
	
	Page* firstpage=m_currentDico->page(0);
	
	
	m_left->showPage(firstpage);
	
	
	//on sauve la page 1 comme tant celle en cours
	setCurrentPage(firstpage);
	//et on montre l'entre en cours de la page 1
	
	setHistoryWriteable(true);
	showCurrentEntry(firstpage);
	setHistoryWriteable(false);
}

void CentralWidget::showDico(int index)
{
	int i;
	int currentsize;
	//Appele quand on clique sur la combo en haut  droite
	//ou plus gnralement quand on passe  un autre dico
	
	m_dicocombo->setCurrentItem(index);	//ncessaire, si ce n'est pas fait directement par un clic
	
	
	//on cache les pages en cours
	//	if(left1) m_left->removePage(left1);
	//	if(left2) m_left->removePage(left2);
	
	if(m_currentDico)
	{
		currentsize=m_currentDico->size();
		for(i=0;i<currentsize;i++) m_left->removePage(m_currentDico->page(i));
	}
	m_currentDico=m_dicos[index];
	//on retient le nom du fichier html qu'il faudra montrer quand on clique sur help
	m_hlpname= m_diconames[index]+".html";
	
	//on montre les nouvelles pages du dico..euh on ne se moque pas :p
	currentsize=m_currentDico->size();
	for(i=0;i<currentsize;i++)
	{
		Page* curpage=m_currentDico->page(i);
		m_left->addTab(curpage,curpage->title());
		
		
		m_left->showPage(curpage);
		
	}
	
	Page* firstpage=m_currentDico->page(0);
	
	
	m_left->showPage(firstpage);
	
	
	//on sauve la page 1 comme tant celle en cours
	setCurrentPage(firstpage);
	//et on montre l'entre en cours de la page 1
	showCurrentEntry(firstpage);
}

void CentralWidget::showDico(QString diconame)
{
	//Mme fonction que la fonction ci dessus, avec comme paramtre le nom
	//de fichier gnrique du dico
	int i = dicoIndex(diconame);
	
	if(i>=0) showDico(i);
	
}

int CentralWidget::dicoIndex(QString diconame)
{
	//Retourne l'index dans la liste de la combobox du dico de nom gnrique
	//diconame
	
	unsigned int i;
	
	strIt it=m_diconames.begin();
	
	for(i=0;it!=m_diconames.end();++it,++i)
	{
		if(m_diconames[i]==diconame) break; 
	}
	
	if(i==m_diconames.size()) return -1;
	else return i;
	
}

void CentralWidget::showCurrentEntry(Page* page)
{
	//affiche l'entre courrente la page Page
	if(page->listBox()->count()>0)
	{
		page->loadItem(page->listBox()->item(page->listBox()->currentItem()));
	}
}

void CentralWidget::pageSelected(QWidget* page)
{
	
	setCurrentPage((Page*)page);
	((Page*)page)->searchField()->setFocus();
	
	
	showCurrentEntry((Page *)page);
	
}


QString CentralWidget::clean(QString entryname)
{
	//Au cas o on trouve des caractres embtants...
	// /me se pose la question de savoir si c'est bien de faire a...
	
	
	int pos = entryname.find("&#");
	int coma = entryname.find(';',pos);
	
	while(pos>=0&&coma>pos) {
		
		entryname.replace(pos,coma-pos+1,QChar(entryname.mid(pos+2,coma-pos-2).toInt()));
		pos = entryname.find("&#");
		coma = entryname.find(';',pos);
	}
	
	return entryname;
}

void CentralWidget::handlehref(const QString &str)
{


	//Oki alors, a c'est costaud c'est pour savoir ce qu'on fait
	//quand on clique sur un lien
	//J'explique pas le code :p
	
	//Voil la norme :
	
	///////// #target :
	//Charger l'entre target dans le dico et la langue en cours
	///////// 1#target :
	//Charger l'entre target dans le dico en cours et dans sa langue 1
	///////// 2#target :
	//Charger l'entre target dans le dico en cours et dans sa langue 2
	///////// nomdelangue_1#target :
	//Charger l'entre target dans le dico de nom gnrique nomdelangue et dans sa langue 1
	///////// nomdelangue_2#target :
	//Charger l'entre target dans le dico de nom gnrique nomdelangue et dans sa langue 2
	
	
	Dico* targetdico;
	Page* targetpage;
	
	int diez = str.find(QChar('#'));
	QString where,what;
	
	if(diez<0)
		return;
	
	where = str.left(diez);
	what=clean(str.right(str.length()-diez-1));
	
	//const char* dbug = what.latin1();
	
	int underskor = where.find(QChar('_'));
	
	if(underskor<0)
	{
		targetdico = m_currentDico;
		
		if(where=="") targetpage = currentPage();
		else
		{
			int numLangue = where.toInt();
			targetpage=m_currentDico->page(numLangue-1);
		}
		if(!targetpage) return;
	}
	else
	{
		QString where_dico = where.left(underskor);
		int where_language = where.right(where.length()-underskor-1).toInt();
		
		int indexdico = dicoIndex(where_dico);
		
		if(indexdico<0) return;
		
		targetdico=m_dicos[indexdico];
		if(!targetdico) return;
		
		targetpage=targetdico->page(where_language-1);
		if(!targetpage) return;
	}
	
	if(targetpage->indexof(what)>=0)
	{
		setHistoryWriteable(true);
		targetpage->setCurrentItem(targetpage->indexof(what));
		setHistoryWriteable(false);
		
		if(m_currentDico!=targetdico) 
			showDico(targetdico->name());

		if(m_currentPage!=targetpage)
		{
			m_left->showPage(targetpage);		
			pageSelected(targetpage);
		}
	}
	
}

void CentralWidget::backward()
{
	if(m_historyCurrent>0)
	{
		m_historyCurrent--;		
		loadHistoryEvent(*m_history[m_historyCurrent]);
		
		if(m_historyCurrent==0) 
		{ 
			backbutton->setEnabled( false );
		}
		if(m_historyCurrent<(signed int)m_history.size()-1)
		{
			forwardbutton->setEnabled( true );
		}
	}
	else
	{
		loadHistoryEvent(*m_history[m_historyCurrent]);
	}
	
	
}

void CentralWidget::forward()
{
	if(m_historyCurrent<(signed int)m_history.size()-1)
	{
		if(m_historyCurrent==0) 
		{
			backbutton->setEnabled( true );
		}
		m_historyCurrent++;
		loadHistoryEvent(*m_history[m_historyCurrent]);
		if(m_historyCurrent==(signed int)m_history.size()-1)
		{
			forwardbutton->setEnabled( false );
		}
	}
}

void CentralWidget::addHistoryEvent(HistoryEvent *he)
{
	if(m_historyCurrent==0) 
	{
		backbutton->setEnabled( true );
	}
	
	forwardbutton->setEnabled( false );
	
	int numbertodelete=m_history.size()-1-m_historyCurrent;
	
	for(int j=0;j<numbertodelete;j++) m_history.pop_back();
	
	m_history.push_back(he);
	m_historyCurrent++;
}

void CentralWidget::loadHistoryEvent(HistoryEvent he)
{
	Dico* targetdico=m_dicos[he.dicoindex];
	Page* targetpage=targetdico->page(he.page-1);
	
	targetpage->loadNumber(he.numberofentry);
	
	if(m_currentDico!=targetdico) showDico(targetdico->name());
	if(m_currentPage!=targetpage)
	{
		
		m_left->showPage(targetpage);
		pageSelected(targetpage);
		
	}
}

void CentralWidget::copy()
{
	if (m_browser->hasSelectedText())
	{
	
	/*
	* On pourrait faire une simple dlgation
	*    m_browser->copy();
	* Mais ce n'est pas juste envers les concepteurs des dicos,
	* autant ajouter la source d'o provient la copie.
	*/
	QString content( "[" + m_dicocombo->currentText() + "]\n" );
	QString selection = m_browser->selectedText();
	QApplication::clipboard()->setText( content + selection );
	}
	// LIMITATIONS
	// la methode selectedText() perd le formattage 'rich text'
	// et wrappe toutes les lignes  80 colonnes. J'ai jet un
	// oeil sur le code interne de Qt (merci Google), et il semble
	// difficile de faire autrement.
}

void CentralWidget::copyTengwar()
{
	if (m_tengwarview->hasSelectedText())
	{
		
		QString selection = m_tengwarview->selectedText();
		QApplication::clipboard()->setText( selection );
	}
	// LIMITATIONS
	// la methode selectedText() perd le formattage 'rich text'
	// et wrappe toutes les lignes  80 colonnes. J'ai jet un
	// oeil sur le code interne de Qt (merci Google), et il semble
	// difficile de faire autrement.
}

void CentralWidget::writeNbWords(int nbwords)
{
	m_statusbar->writeNbWords(nbwords);
}

void CentralWidget::setCurrentPage(Page* page)
{
	m_currentPage=page;
	writeNbWords(m_currentPage->nbWords());
}

/* EOF */
