/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifndef DP_CENTRALWIDGET_H
#define DP_CENTRALWIDGET_H

#include <qwidget.h>
#include <qtextbrowser.h>
#include <qsplitter.h>

#include <vector>
using namespace std;

#include <qvbox.h>
#include <qlistbox.h>
#include <qlist.h>
#include <qpixmap.h>
#include <qtextview.h>

#include "page.h"
#include "Dico.h"
#include "mainwindow.h"
#include "teng_transcription.h"

class QVBoxLayout;
class QHBoxLayout;
class QTabWidget;
class ListBoxManager;
class QPushButton;
class QLineEdit;
class QListBoxItem;
class QLabel;
class QComboBox;
class Entry;
class MyBrowser;
class SentenceLine;
class DFStatusBar;
class SearchInterface;

typedef struct HistoryEvent{int dicoindex;int page;int numberofentry;}HE;


class CentralWidget : public QSplitter
{
    Q_OBJECT

  public:
	QString m_hlpname;

    CentralWidget( MainWindow* mainwindow, DFStatusBar* dfstatusbar, const char *name = 0 );

	MainWindow* mainWindow() {return m_mainwindow;};
	QTabWidget* TabWidget() {return m_left;};
	void loadModes();
    void loadDicos();
	void loadDico(QString diconame);
	void loadEntry(Entry *entry);
	void showCurrentEntry(Page* page);
	void setCurrentPage(Page* page) ;
	Page* currentPage() {return m_currentPage;};
	MyBrowser* browser() {return m_browser;};
/*	QTextView* tengView() {return m_tengwarview;}; MODIF */ QTextBrowser* tengView() {return m_tengwarview;}; 
	SentenceLine* sentenceLine() {return m_sentenceline;};
	int dicoIndex(QString diconame);
	QString helpfilename(void) {return m_hlpname;};
	void loadHistoryEvent(HistoryEvent heGrenade);
	bool isHistoryWriteable() {return m_historyWriteable;};
	void setHistoryWriteable(bool oki) {m_historyWriteable=oki;};
	void addHistoryEvent(HistoryEvent *heGrenade);
	void writeNbWords(int nbwords);
	bool built() {return m_bBuilt;};
	void setBuilt(bool built) {m_bBuilt=true;};
	CTranscription* transcriptor(int i) {return m_modes[i];};

  public slots :
	void dicoClickedSelected(int index);
//	void pageClickedSelected(QWidget* page);
	void showDico(int index);
	void showDico(QString diconame);
	void pageSelected(QWidget* page);
	void handlehref(const QString &str);
	void backward();
	void forward();
	void copy( void );
	void copyTengwar( void );

  protected:
    void setupListBox();
	QString clean(QString entryname);

	bool m_bBuilt;

	MainWindow* m_mainwindow;

	QListBoxItem *currentDisplayed;

    QHBoxLayout *mainGrid;
	
	Dico*		m_currentDico;
	Page*		m_currentPage;

	DFStatusBar* m_statusbar;

	QPushButton* backbutton;
	QPushButton* forwardbutton;

	QPixmap *backpix;
	QPixmap *forwpix;
	QPixmap *backtranspix;	
	QPixmap *forwtranspix;

	QVBox *right;

	QTabWidget* m_left;

	MyBrowser* m_browser;
/*	QTextView* m_tengwarview; MODIF */  QTextBrowser* m_tengwarview;
	QComboBox* m_dicocombo;

	SentenceLine* m_sentenceline;

	vector<HistoryEvent *> m_history;
	bool m_historyWriteable;
	int m_historyCurrent;

	vector<QString>m_diconames;
	vector<Dico*>m_dicos;
	vector<QString>m_modenames;
	vector<CTranscription*>m_modes;
};

#endif
