/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#include <qapplication.h>
#include <qsound.h>
#include <qstrlist.h>

#ifdef WIN32
#include <winsock.h>
#else
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#endif

#include "dictProtocol.h"

void DICTClient::connectToHost( QString host, unsigned short port )
{
	struct hostent *ptrh;  
	struct protoent *ptrp; 
	struct sockaddr_in sad;
	int    option_value;
	
	/*
	 * Some Windoze-specific garbage first, to activate the socket layer.
	 * BLARG!!!
	 */
#ifdef WIN32
	WSADATA info; 
	if (WSAStartup(MAKEWORD(1,1), &info) != 0) 
	{
		sd = -666; /* The Hell with it */
		return;
	}
#endif

	/* 
	 * Let's establish a TCP connection now, in a pure traditional way
	 */
	memset( (char *)&sad, 0, sizeof(sad) ); 
	sad.sin_family = AF_INET; 
	sad.sin_port   = htons( (unsigned short) port );
  
	ptrh = gethostbyname( host.ascii() );
	if (((char *)ptrh) == NULL) 
	{
		sd = -1;
		return;
	}
	memcpy( &sad.sin_addr, ptrh->h_addr, ptrh->h_length );
  
	if (((int)(ptrp = getprotobyname("tcp"))) == 0) 
	{
		sd = -2;
		return;
	}
  
	sd = socket( PF_INET, SOCK_STREAM, ptrp->p_proto);
	if (sd < 0) 
	{
		sd = -3;
		return;
	}
  
	option_value = 1;
	if (setsockopt( sd, SOL_SOCKET, SO_REUSEADDR, 
		 (char *)&option_value, sizeof(option_value) ) < 0) 
	{
		/* Failed to set the socket in 'reuse' mode... 
		 * Let's ignore this error.
		 */
	}
  
	if (connect(sd, (struct sockaddr *)&sad, sizeof(sad)) < 0) 
	{
		sd = -4;
		return;
	}
}

QString DICTClient::lookup( const char *word, const char *book ) 
{
	char	buffer[1024];
	int		n     = 0;
	int		count = 0;
	QString definition;
  
	/* Construct the query to send to the server */
	QString query = QString("DEFINE ") + book + " \"" + word + "\"\n\r";

  	if (send( sd, query.ascii(), query.length(), 0 ) < 0) 
	{
		sd = -1000;
		return( "Not connected" );
	}
	  
	while ( (n = readline( buffer, sizeof(buffer) )) > 0)
	{
		QString buf( buffer );

		if (buf.startsWith( "42" ))
			break;
    
		if (buf.startsWith( "220" ))
			continue;	/* Ignore successful connection message */
    
		if (buf.startsWith( "250" ))
			break;		/* break out if completed successfully */
    
		if (buf.startsWith( "150" ))
		{
			/* How many matches */
			/* numOfMatches = atoi(buf+3); */
			continue;
		}

		/* Ignore EO each entry from server */
		if (buf.startsWith( "." )) 
		{
			count++;
			continue;
		}
    
		/* Definition found */
		if (buf.startsWith( "151" ))
		{
			if (count > 0)
			{
				definition.append( "\n" );
			}

			/* Extract dictionary name */
			unsigned i = 0;
			int      j = 0;
			QString bk;
			for ( i = strlen(buffer) -1, j = 0; i > 0; i--) 
			{
				if ((buffer[i] == '"') && (j == 0))
				{
					j = 1;
					continue;
				}
				else if (buffer[i] == '"') 
				{
					bk = &buffer[i];
					break;
				}
			}
			definition.append( "--@From " ).append( bk ).append( "@--" );
			continue;
		}

		if (buf.startsWith( "501" ))
			break;		/* 501 syntax error - break out */

		if (buf.startsWith( "550" ))
			break;		/* 550 invalid database - break out */

		if (buf.startsWith( "552" ))
			break;		/* 552 no match - break out */
    
		definition.append( buf );
	}
	return( definition );
}

QStrList DICTClient::showdb() 
{
	char	buffer[1024];
	int		n       = 0;
	int		count   = 0;
	bool    web1913 = false;
	QStrList db;
  
	/* Construct the query to send to the server */
	QString query = QString("SHOW DB\n\r");

  	if (send( sd, query.ascii(), query.length(), 0 ) < 0) 
	{
		sd = -1000;
		return( "Not connected" );
	}
	  
	while ( (n = readline( buffer, sizeof(buffer) )) > 0)
	{
		QString buf( buffer );

		if (buf.startsWith( "42" ))
			break;
    
		if (buf.startsWith( "220" ))
			continue;	/* Ignore successful connection message */
    
		if (buf.startsWith( "250" ))
			break;		/* break out if completed successfully */
		
		if (buf.startsWith( "110" ))
			continue;	/* Ignore (how many database) */

		/* Ignore EO each entry from server */
		if (buf.startsWith( "." )) 
		{
		  count++;
		  continue;
		}

		if (buf.startsWith( "501" ))
			break;		/* 501 syntax error - break out */

		/* --- Database --- */
		if (buf.startsWith( "web" )) /* Webster */
		{
			db.insert( 0, buf.stripWhiteSpace() );
			web1913 = true;
		}
		else if (buf.startsWith( "wn" )) /* Wordnet */
		{
			db.insert( web1913 ? 1 : 0, buf.stripWhiteSpace() );
		}
		else
		{
			db.append( buf.stripWhiteSpace() );
		}
	}
	return( db );
}

int DICTClient::readline( char *buffer, int maxlen )
{
	int i, nc;
	char c;

	for ( i=0; i < maxlen; i++ )
	{
		if ((nc = recv( sd, &c, 1, 0 )) == 1) 
		{
			*buffer++ = c;
			if (c == '\n') 
				break;
		}
		else if (nc == 0) 
		{
			if (i == 1)
				return 0;
			else
				break;
		}
	}
	*buffer = '\0';
	return i;
}

void DICTClient::run()
{
	/*
	 * This is the function executed when the thread runs.
	 */
	QString   word = searchedWord;
	QString   dict = searchedDict;
	QString   definition;
	QStrList  databases;
	
	if (sd < 0)
	{
		/*
		 * Establish a permanent connection.
		 */
		connectToHost( host );
		if (sd >= 0)
		{
			/*
			 * Retrieve the available databases.
			 */
			databases = showdb();
			qApp->postEvent( owner, new DICTClientEvent( databases ));
		}
		else
		{
			definition = "Not connected";
		}
	}

	if (word != "")
	{
		if (sd >= 0)
		{
			/*
			 * Connection ok, search our entry.
			 */
			definition = lookup( word, dict ); 
			if (sd < 0)
			{
				/* 
				 * Got deconnected? Let's retry, perhaps the server shut
				 * down our connection on inactivity time-out.
				 */
				connectToHost( host );
				definition = lookup( word, dict ); 
			}
		}
	
		if (!QSound::available())
		{
			qApp->beep();
		}
		else
		{
			QSound::play( qApp->applicationDirPath()+"/../sounds/beep.wav" );
		}
	}

	qApp->postEvent( owner, new DICTClientEvent( definition ));
}

void DICTClient::define( QString word, QString dict )
{
	/*
	 * Asynchronous dictionary look-up.
	 */
	searchedWord = word;
	searchedDict = dict;
	this->start();
}

/* EOF */
