/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#ifndef DICTPROTOCOL_H
#define DICTPROTOCOL_H

#include <qthread.h> 
#include <qwidget.h> 
#include <qstrlist.h> 

class DICTClient : public QThread 
{
  public:
	DICTClient( QWidget *parent, QString mhost = "dict.org" ) 
		: QThread(), host( mhost ), sd( -1 ), owner( parent ) {}

	void DICTClient::define( QString word, QString dict = "*" );
  
  protected:
	QString host;
	int     sd;
	QWidget *owner;
	QString searchedWord;
	QString searchedDict;

  protected:
	virtual void run();

  	void connectToHost( QString host, unsigned short port = 2628 );
	int readline( char *buffer, int maxlen );
	QString lookup( const char *word, const char *book = "*" );
	QStrList showdb();
};

class DICTClientEvent : public QCustomEvent
{
  public:
	enum Type {
        Definition = (QEvent::User + 20),
		Database
	};

  public:
    DICTClientEvent( QString def )
        : QCustomEvent( Definition ), definition( def ) {};
    DICTClientEvent( QStrList datab )
        : QCustomEvent( Database ), db( datab ) {};

	inline QString result() const { return( definition ); };
	inline QStrList database() const { return( db ); };

  protected:
    QString definition;
	QStrList db;
};

#endif
