/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include <qapplication.h>
#include <qmessagebox.h>
#include <qsound.h>
#include <stdio.h>
#include "httpSynchronizer.h"
#include "preferences.h"

extern int httpSynchronize( const char *lpcSite,
				            const char *lpcURLBase,
					        const char *lpcDirBase );

void httpSynchronizer::run()
{
	int count = 0;
	
	try
	{
		QString address = DFConfig::instance()->getString( "httpSynchronizer", "address" );
		QString source  = DFConfig::instance()->getString( "httpSynchronizer", "source"  );
		count = httpSynchronize( address, source, "dicos" );
	}
	catch (...)
	{
		/* 
		 * Des fois qu'on crashe sur la synchronisation, je ne suis pas sr 
		 * 100% de mon code avec les API Windoze.
		 */
		count = -1;
	}

	/*
	 * Ca se passe trs mal (crash ou blocage) si on tente d'afficher une popup 
	 * de rsultat depuis ce thread et que l'on est en train de naviguer dans 
	 * un menu (je suppose que a interagit avec la boucle d'vnement du menu).
	 * Donc on rpond par un vnement asynchrone, et c'est la fentre principale
	 * qui se chargera d'afficher les rsultats.
	 */
	qApp->postEvent( qApp->mainWidget(), new httpSynchronizerEvent( count ));

	if (!QSound::available())
	{
		qApp->beep();
	}
	else
	{
		QSound::play( qApp->applicationDirPath()+"/../sounds/beep.wav" );
	}

	/*
	 * Profitons du mcanisme pour tre vicieux et n'acquitter que tardivement
	 * la mise  jour, afin que les utilisateurs ne puissent pas surcharger
	 * le site de requtes web en rafale.
	 */
	this->sleep( 30 ); // 30 bonnes secondes
	qApp->postEvent( qApp->mainWidget(), new httpSynchronizerEvent());
}

/* EOF */
