/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

//C'est la fentre principale.

#include "mainwindow.h"
#include "centralwidget.h"
#include "helpwindow.h"
#include "about.h"
#include "dictOnline.h"
#include "searchinterface.h"
#include "preferences.h"
#include <qtextstream.h> 

#include <qsound.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qapplication.h>
#include <qfile.h>
#include "dfstatusbar.h"
#include "downloadbar.h"


#include <vector>
using namespace std;

MainWindow::MainWindow()
    : QMainWindow( 0, "Dico Viewer application" ),
      filename( QString::null )
{
	m_statusbar = new DFStatusBar(this);

	styleCDE();
	//On set le titre
	setCaption( "[Free software] - Dragon Flame" );  
  
	//On set le gros widget central, qui contient la plupart
	//des composants de l'appli
    setupCentralWidget();

	 //On set la barre de menus
	setupMenuBar();

	QString dictOnlineAddress = DFConfig::instance()->getString( "dictOnline", "address", "dict.org" );
	m_dictOnline = new DictOnline( dictOnlineAddress, this );
	m_dictOnline->setCaption("[Free software] - On-line Dictionaries");

	m_si=new SearchInterface(this);
	
	
	m_hlpw = new HelpWindow(view,"", ".", 0, "help viewer");
    m_hlpw->setCaption("[Free software] - Documentation");

	m_about= new About(this);
	m_about->setCaption("[Free software] - About Dragon Flame");

	show();
}

void MainWindow::setupMenuBar()
{
	//allons-y pour la barre des menus

	//menu file
    m_file = new QPopupMenu( this );
    menuBar()->insertItem( "&File", m_file );
    m_file->insertItem( "Quit", this , SLOT( quit() ), CTRL + Key_Q );

	m_edit = new QPopupMenu( this );
	menuBar()->insertItem( "&Edit", m_edit);
	m_edit->insertItem( "Copy selected text", this, SLOT( copy() ), CTRL + Key_S  );
	m_edit->insertItem( "Copy selected tengwar", this, SLOT( copyTengwar() ), CTRL + Key_T  );
	m_edit->insertSeparator();
	m_edit->insertItem( "Search for a regular expression", this, SLOT( searchExp() ), CTRL + Key_F  );

	m_tools = new QPopupMenu( this );
	menuBar()->insertItem( "&Tools", m_tools);
	int id_dictOnline = m_tools->insertItem( "Search Online Dictionnaries", this, SLOT( searchDictOnline() ), CTRL + Key_O  );
	m_tools->setItemEnabled( id_dictOnline, 
		                     DFConfig::instance()->getBool( "dictOnline", "enable", false ) );
    id_checkUpdates = m_tools->insertItem( "Check dictionary updates...", this, SLOT( updates() ));

	m_tools->setItemEnabled( id_checkUpdates, 
							DFConfig::instance()->getBool( "httpSynchronizer", "enable", false ));


	//menu help
	m_help = new QPopupMenu( this );
    menuBar()->insertItem( "&Help", m_help );
	m_help->insertItem( "Help on current dictionnary", this, SLOT( help() ), CTRL + Key_H );
	m_help->insertSeparator();
	m_help->insertItem( "Dragon Flame Documentation", this, SLOT( documentation() ), CTRL + Key_D );
    m_help->insertItem( "About...", this, SLOT( about() ), CTRL + Key_A );

	//la toolbar
/*	QToolBar*  fileTools = new QToolBar( this, "file operations" );
    fileTools->setLabel( tr( "File Operations" ) );
    QToolButton * fileOpen
	= new QToolButton( QPixmap( "images/fileopen.xpm" ), "Open File", QString::null,
			   this, SLOT(fileOpen()), fileTools, "open file" );

*/
	setToolBarsMovable(false);
	resize(640,480);
}

void MainWindow::setupCentralWidget()
{
	//on set le central widget (cf le fichier du centralwidget)
	//attention, il est gros.
    view = new CentralWidget( this,m_statusbar,"centralwidget" );
    setCentralWidget( view );

}

void MainWindow::help()
{
	//appel quand on clique sur Help->help
	//voir la connexion des signaux ci dessus

	m_hlpw->showe();
}

void MainWindow::about()
{
	//appel quand on clique sur Help->about
	//voir la connexion des signaux ci dessus
	QSound::play( qApp->applicationDirPath()+"/../sounds/bdragon2.wav" );

	m_about->show();
}

void MainWindow::updates()
{	

	/*
	 * Appel quand on clique "Help / Check updates"
	 */
	if (m_updates.running())
	{
		/*
		 * Si une mise  jour est dj en cours, ignorer
		 * une nouvelle demande. C'est une prcaution a priori
		 * inutile, parce que l'option de menu est grise un peu 
		 * plus bas, mais sait-on jamais.
		 */
		QMessageBox::warning( this, 
		  "Dragon Flame",
		  "<qt><b>Dictionary update:</b><br/>"
		  "Please wait, an update is already in progresss</qt>",
		  "Close" );
		return;
	}

	int action = QMessageBox::information( this, 
		  "Dragon Flame",
		  "<qt><b>Dictionary update:</b><br/>"
		  "Do you want to check for dictionary updates "
		  "on the Internet (<i>first ensure your internet "
		  "connection is active</i>) ?</qt>",
		  "Yes", "No", 0, 0, 1 );

	if (0 == action) // Yes
	{
		/*
		 * La connexion internet peut tre longue et pourrait
		 * bloquer l'application pendant un temps non ngligeable. 
		 * Pour viter cela, on effectue les traitements dans
		 * un thread spar. Et puis...
		 */
		m_statusbar->start();
		m_tools->setItemEnabled( id_checkUpdates, false );
		m_updates.start();
	}
}

void MainWindow::customEvent( QCustomEvent *e )
{
	/*
	 * ... Et puis c'est l'occasion de faire un peu de programmation
	 * vnementielle pour le rsultat.
	 */
	if ( e->type() == static_cast<QEvent::Type>(httpSynchronizerEvent::Done)) 
	{
		m_statusbar->stop();
		httpSynchronizerEvent* updateResult = (httpSynchronizerEvent *)e;

		switch ( updateResult->count() )
		{
		  case -1:
			QMessageBox::warning( 
				this, 
				"Dragon Flame",
				"<qt><b>Dictionary update:</b><br/>"
				"Internet connection failure.</qt>",
				"Close" );
			break;

		  case 0:
			QMessageBox::information( 
				this, 
				"Dragon Flame",
				"<qt><b>Dictionary update:</b><br/>"
				"No new dictionary files retrieved. Either there is no new version "
				"available, or your internet connexion is not set up properly.</qt>",
				"Close" );
			break;

		  default:
			char tmp[200];
			sprintf( tmp, "<qt><b>Dictionary update:</b><br/>"
				"%d files updated.<br/>Restart <i>Dragon Flame</i> to take them into account.</qt>",
				updateResult->count() );
			QMessageBox::information( 
				this, 
				"Dragon Flame",
				tmp,
				"Close" );
			break;
		}	
	}
	else if ( e->type() == static_cast<QEvent::Type>(httpSynchronizerEvent::Ready)) 
	{
		m_tools->setItemEnabled( id_checkUpdates, true );
	}
	
}

void MainWindow::quit()
{
	QApplication::exit( 0 );
}

void MainWindow::closeEvent( QCloseEvent *)
{
	//surcharg, parce qu'on veut que le type puisse pas faire une connerie
	//quand il clique sur la petite croix en haut  droite

	 QApplication::exit( 0 );
}

void MainWindow::styleCDE()
{
//	qApp->setStyle( new QCDEStyle( TRUE ) );

    QPalette p = qApp->palette();
    p.setColor( QPalette::Active, QColorGroup::Base, QColor( 230, 230, 230  ) );
    p.setColor( QPalette::Inactive, QColorGroup::Base, QColor( 230, 230, 230) );
    p.setColor( QPalette::Disabled, QColorGroup::Base, QColor( 230, 230, 230) );
   // p.setColor( QPalette::Active, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Active, QColorGroup::HighlightedText, QColor( 230, 230, 230 ) );
  //  p.setColor( QPalette::Inactive, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Inactive, QColorGroup::HighlightedText, QColor(230, 230, 230) );
  //  p.setColor( QPalette::Disabled, QColorGroup::Highlight, Qt::white );
    p.setColor( QPalette::Disabled, QColorGroup::HighlightedText, QColor( 230, 230, 230) );
  //  p.setColor( QPalette::Active, QColorGroup::Foreground, Qt::black );
  //  p.setColor( QPalette::Active, QColorGroup::Text, Qt::black );
   // p.setColor( QPalette::Active, QColorGroup::ButtonText, Qt::white );
   // p.setColor( QPalette::Inactive, QColorGroup::Foreground, Qt::black );
  //  p.setColor( QPalette::Inactive, QColorGroup::Text, Qt::black );
  //  p.setColor( QPalette::Inactive, QColorGroup::ButtonText, Qt::black );
  //  p.setColor( QPalette::Disabled, QColorGroup::Foreground, Qt::lightGray );
  //  p.setColor( QPalette::Disabled, QColorGroup::Text, Qt::lightGray );
  //  p.setColor( QPalette::Disabled, QColorGroup::ButtonText, Qt::lightGray );
    qApp->setPalette( p, TRUE );
 //   qApp->setFont( QFont( "times", appFont.pointSize() ), TRUE );
}

void MainWindow::copy()
{
	view->copy();
}

void MainWindow::copyTengwar()
{
	view->copyTengwar();
}

void MainWindow::searchExp()
{
	m_si->popUp();
}

void MainWindow::searchDictOnline()
{
   m_dictOnline->define( view->currentPage()->searchField()->text() );
}

void MainWindow::documentation()
{
   m_hlpw->showd();
}
