/*
* Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
*
* This program is free software; you can redistribute it and/or
* modify it under the terms of the GNU General Public License
* as published by the Free Software Foundation; either version 2
* of the License, or (at your option) any later version.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.
* 
* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software Foundation, 
* Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*
*/
#include "page.h"
#include "centralwidget.h"
#include "mybrowser.h"
#include "sentenceline.h"
#include "searchinterface.h"

#include <qlabel.h>
#include <qmessagebox.h>

Page::Page( CentralWidget *cwidg, Dico *mydico, QWidget *parent, 
		    const char *, WFlags, bool) : QVBox( parent )
{
	m_cwidg		=	cwidg;
	m_tbw		=	cwidg->TabWidget();
	m_browser	=	m_cwidg->browser();
	m_mydico	=	mydico;
	
	search = new SearchField (this,this);
	
	//l, on cre la listbox des mots contenus dans le dictionnaire
	qlb = new QListBox( this );
	connect( qlb, SIGNAL( currentChanged (QListBoxItem *) ),this, SLOT( loadCurrentWithoutUpdatingSearchField() )  );	
	connect( qlb, SIGNAL( pressed (QListBoxItem *) ),this, SLOT( loadItemNReformat(QListBoxItem *) )  );	

	// Je commente cette ligne parce que le signal "enter" est gr par le searchfield de la page
	// Du coup, y'avait conflit (fonction appele deux fois, et l'historique n'aime pas)
	//connect( qlb, SIGNAL( returnPressed(QListBoxItem *) ),this, SLOT( loadItemNReformat(QListBoxItem *) )  );
	
	search->attributeListBox( qlb );
	qlb->setFocusPolicy( QWidget::NoFocus );
	
	//et on cre aussi un listboxmanager, qui se greffe dessus
   	qlbm = new ListBoxManager( qlb );
	//c'est un conteneur qui permet de grer un peu plus facilement ce qu'il
	//y'a dans la listbox
	
	QLabel *coucou = new QLabel( "Tengwar mode :", this );
	coucou->setMargin( 2 );
	m_choosemodebox = new QComboBox( this );
	connect( m_choosemodebox, SIGNAL( activated(int) ),this, SLOT( modeHasChanged(int) )  );
	m_choosemodebox->setFocusPolicy( QWidget::NoFocus );
	
	QString unifontname = DFConfig::instance()->getString( "fonts", "unicode", "Lucida Sans Unicode" );
	QFont unifont( unifontname, 8 );	
	search->setFont( unifont );
	qlb->setFont( unifont );
	
	voisin = NULL;
};

void Page::modeHasChanged(int selected)
{
	m_mode = selected;
	if (m_cwidg->sentenceLine()->hasFocus()) 
	{
		m_cwidg->sentenceLine()->tengwarRender();
	}
	else
	{
		tengwarRender(search->text());
	}
}

void Page::setModeAndModeNames( QString modename, vector<QString> modenames )
{
	vector<QString>::iterator it = modenames.begin();
	
	for (int i=0; it!=modenames.end(); ++it,++i)
	{
		m_choosemodebox->insertItem( modenames[i] );
	}
	
	m_choosemodebox->setCurrentItem( 0 );
	
	for (int j=0; j<m_choosemodebox->count(); j++)
	{
		if(modename==m_choosemodebox->text( j ))
		{
			m_mode = j;
			m_choosemodebox->setCurrentItem( j );
			break;
		}
	}
}

void Page::loadEntry( Entry *entry )
{
	//charge telle entre
	if (m_cwidg->isHistoryWriteable())
	{
		HistoryEvent *he = new HistoryEvent();
		he->dicoindex = m_cwidg->dicoIndex( m_mydico->name() );
		he->page = m_numberLangue;
		he->numberofentry = qlbm->indexof( entry );
		
		m_cwidg->addHistoryEvent( he );
	}

	// entry->getHtml() pour avoir le contenu HTML
	m_browser->setText( entry->getHtml() );
	tengwarRender( entry->getName() );
}

void Page::loadCurrentPub( void )
{
	//charge l'entre en cours
	if( qlb->count()>0 )
	{
		QListBoxItem *item = qlb->item( qlb->currentItem() );
		loadItem( item );
		search->setText( item->text() );
		//	tengwarRender();
	}
}

void Page::loadcurrent( void )
{
	//charge l'entre en cours
	if ( qlb->count()>0 )
	{
		QListBoxItem *item = qlb->item( qlb->currentItem() );
		loadItem(item);
		search->setText(item->text());
		//	tengwarRender();
	}
}

void Page::loadCurrentWithoutUpdatingSearchField(void)
{
	//charge l'entre en cours
	if ( qlb->count()>0 )
	{
		QListBoxItem *item = qlb->item( qlb->currentItem() );
		loadItem(item);
		//search->setText(item->text());
		//	tengwarRender();
	}
}

void Page::loadZero( void )
{
	if ( qlb->count()>0 )
	{
		qlb->setCurrentItem( 0 );
		qlb->setTopItem( 0 );
		//	tengwarRender(qlbm->entryAt(0)->getName());
		//	loadEntry(qlbm->entryAt(0));
	}
}

void Page::loadItem( QListBoxItem *item )
{	
	if (item)
	{
		loadEntry(qlbm->entryAt( qlb->index(item)) );
	}
}

void Page::loadItemNReformat( QListBoxItem *item )
{
	if (item)
	{
		search->setFocus();
		m_cwidg->setHistoryWriteable( true );
		loadEntry(qlbm->entryAt(qlb->index( item )));
		search->setText( item->text() );
		//	tengwarRender();
		m_cwidg->setHistoryWriteable( false );
	}
}


void Page::textHasChanged()
{
	goodPosition();
	tengwarRender( search->text() );
}

void Page::goodPosition()
{
	
	//a, c'est la fonction qui slectionne la bonne entre quand on tape
	//un dbut de mot dans la line edit search
	bool stop = false;
	int i = qlb->currentItem();
	
	QString text=search->text();
	QString current;
	QString nextone;
	
	if( qlb->count() <= 1 ) return;
	
	if (text == "")
	{
		qlb->setSelected( 0, true );
		qlb->setTopItem( 0 );
		return;
	}
	
	while( i>0 )
	{
		if ( qlbm->infEq(text,qlb->item(i)->text()) )
		{
			i--;
		}
		else break;
	}
	//ramne le curseur un peu plus haut
	//dans la liste pour pouvoir la redescendre aprs
	while( i< (signed int)(qlb->count()-1) && !stop )
	{
		current = qlb->item(i)->text();
		nextone = qlb->item(i+1)->text();
		
		if (qlbm->inf(text,nextone) && qlbm->supEq(text,current) || qlbm->infEq(text,current))
		{
			if ( qlbm->sup(text,current) && qlbm->startsWith(nextone,text) )
			{
				i++;
			}
			stop = true;
		}
		i++;
	}
	
	if (stop == false) i++;
	i--;
	
	qlb->setSelected( i,true );
	qlb->setTopItem( i );
	
	
//	if ( qlb->text(i)==text )
	//{
		//criture dans l'historique commente
		//trop nervant, l	
		//	m_cwidg->setHistoryWriteable(true);
	//	loadCurrentWithoutUpdatingSearchField();
		//	m_cwidg->setHistoryWriteable(false);
	//}
}

void Page::loadNumber( int i )
{
	if( (signed int)qlb->count()>i )
	{
		qlb->setCurrentItem( i );
		qlb->setTopItem( i );
		loadcurrent();
	}
}


void Page::setCurrentItem( int i )
{
	if( (signed int)qlb->count()>i )
	{
		qlb->setCurrentItem( i );
		qlb->setTopItem( i);
	}
}

int Page::indexof( QString word )
{
	return qlbm->indexof( word );
}

int Page::nbWords()
{
	return qlb->count();
}

void Page::tengwarRender( QString toaffiche )
{
	toaffiche.replace ( QRegExp(QChar(375)), "^y" );
	toaffiche.replace ( QRegExp(QChar(339)), "e" );
	
	
	QString towrite;
	if(m_choosemodebox->currentItem()==0)
		towrite="<center><font face=\"Lucida Sans Unicode\" color=#660000>"+toaffiche+"</font></center>";
	else
		towrite="<center><font color=#660000>"+QString(m_cwidg->transcriptor(m_choosemodebox->currentItem()-1)->Roman2Tengwar(toaffiche.latin1()))+"</font></center>";
	
	m_cwidg->tengView()->setText(towrite);
	
	//	const char *forsee = towrite.latin1();
	//m_cwidg->tengView()->ensureVisible(0,25);
	//	char* test =m_mode.latin1();
}

void Page::searchExp( QRegExp exp, bool casesens, bool inentrynames )
{
	int		startindex		=	qlb->currentItem();
	bool	cyclestarted	=	false;
	bool	founded			=	false;
	int		foundedindex =0; //initialis pour viter un warning
	int		indexcounter = 0;
	QString toaffiche;

	exp.setCaseSensitive(casesens);
	
	if(exp == m_srchLastExp
		&&	startindex	 == m_srchLastIndex
		&&	casesens	 == m_srchLastCaseSens
		&&	inentrynames == m_srchLastinEntryNames)
		startindex++;

	if(startindex==(signed int)qlb->count()) startindex = 0; // TODO voir si on ne peut pas plutt caster startindex en unsigned
		
	indexcounter       = startindex;		
	m_srchLastExp      = exp;
	m_srchLastCaseSens = casesens;
	m_srchLastinEntryNames = inentrynames;

	if(!inentrynames)
	{	
		while((startindex!=indexcounter||!cyclestarted)&&!founded)
		{
			toaffiche = qlbm->entryAt(indexcounter)->getHtml();
			
			if(findExpAndColorizeSearchString(toaffiche,exp))
			{
				foundedindex=indexcounter;
				founded=true;		
			}
			if(!cyclestarted) cyclestarted = true;
			indexcounter++;
			if(indexcounter==(signed int)qlb->count()) indexcounter = 0; // TODO idem
		}
		
		if(founded)
		{
			m_srchLastIndex=foundedindex;
			qlb->setCurrentItem(foundedindex);
			qlb->setTopItem(foundedindex);
			loadItemNReformat(qlb->item(foundedindex));
			m_browser->setText(toaffiche);
		}
		else
		{
			QMessageBox::information( m_cwidg->mainWindow()->searchInterface(), "Bad Roll !","This expression was not found.\n" );
		}
	}
	else
	{
		while((startindex!=indexcounter||!cyclestarted)&&!founded)
		{
			toaffiche = QString(" ")+qlbm->entryAt(indexcounter)->getName()+" ";
			
			if(toaffiche.find(exp)>=0)
			{
				foundedindex=indexcounter;
				founded=true;	
			}
			if(!cyclestarted) cyclestarted = true;
			indexcounter++;
			if(indexcounter==(signed int)qlb->count()) indexcounter = 0; //TODO pareil
		}
		
		if(founded)
		{
			m_srchLastIndex=foundedindex;
			qlb->setCurrentItem(foundedindex);
			qlb->setTopItem(foundedindex);
			loadItemNReformat(qlb->item(foundedindex));
		}
		else
		{
			QMessageBox::information( m_cwidg->mainWindow()->searchInterface(), "Bad Roll !","This expression was not found.\n" );
		}

	}
}

bool Page::findExpAndColorizeSearchString(QString& toaffiche,QRegExp exp)
{
	int len=toaffiche.length();
	int* reperes;
	bool founded = false;
	bool writing=true;
	QString htmlCleaned="";
	
	reperes = new int[len];
	int repcursor=0;
	
	/*premire partie : extraire le texte de l'html et garder les repres*/
	for(int i=0;i<len;i++)
	{
		if(writing)
		{
			if(toaffiche[i]=='&'
				&&toaffiche[i+1]=='n'
				&&toaffiche[i+2]=='b'
				&&toaffiche[i+3]=='s'
				&&toaffiche[i+4]=='p'
				&&toaffiche[i+5]==';')
			{
				i+=5;
			}
			else if(toaffiche[i]=='<')
			{
				writing=false;
			}
			else
			{
				htmlCleaned+=toaffiche[i];
				reperes[repcursor]=i;
				repcursor++;
			}
		}
		else
		{
			if(toaffiche[i]=='>')
			{
				writing=true;
			}
		}
	}
	
	/*deuxime partie : reprer les regexp*/
	int	newpos = 0;
	int lastpos = -1;
	int decalage = 0;
	int foundedlength;
	
	int newposcorresp;
	int lengthcorresp;
	
	newpos=exp.match (htmlCleaned, lastpos+1, &foundedlength, true );
	
	while(newpos!=-1)
	{
		if(!founded) founded=true;
		
		lastpos = newpos+foundedlength-1;
		
		newposcorresp=reperes[newpos]+decalage;
		lengthcorresp=reperes[newpos+foundedlength-1]-newposcorresp+1+decalage;
		
		decalage+=colorizeSearchString(toaffiche,newposcorresp,lengthcorresp);
		
		newpos=exp.match (htmlCleaned, lastpos+1, &foundedlength, true );
	}
	return founded;
}

int Page::colorizeSearchString(QString &toColorize,int start, int len)
{
	int startlength = toColorize.length();
	
	QString zoneToColorize = toColorize.mid(start,len);
	QString colorizedZone="<font color=red>";
	
	int i;
	
	for(i=0;i<len;i++)
	{
		if(zoneToColorize[i]=='<') colorizedZone+="</font><";
		else if(zoneToColorize[i]=='>') colorizedZone+="><font color=red>";
		else colorizedZone+=zoneToColorize[i];
	}
	
	colorizedZone+="</font>";
	
	toColorize.replace(start,len,colorizedZone);
	
	return (toColorize.length()-startlength);
}

/* EOF */
