/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#ifndef PAGE_H
#define PAGE_H



#include "entry.h"
#include "listboxmanager.h"
#include "searchfield.h"
#include "Dico.h" 


#include <qvbox.h>
#include <qlistbox.h>
#include <qtextbrowser.h>
#include <qlineedit.h>
#include <qtabwidget.h>
#include <qcombobox.h>
#include <qregexp.h> //new


class SearchField;
class CentralWidget;
class MyBrowser;

class Page : public QVBox
{
	Q_OBJECT

	CentralWidget* m_cwidg;
	QTabWidget* m_tbw;
	MyBrowser* m_browser;
	
	SearchField *search;
	QListBox *qlb;
	ListBoxManager *qlbm;

	QComboBox* m_choosemodebox;

	QString m_title;
	QString m_fileName;
	int m_mode;

	Page* voisin;

	int m_numberLangue;
	Dico* m_mydico;


	QRegExp m_srchLastExp;
	int m_srchLastIndex;
	bool m_srchLastCaseSens;
	bool m_srchLastinEntryNames;

	void setFileName(QString fileName) {m_fileName=fileName;};
	void setTitle(QString title) {m_title=title;};
	void setNumberLangue(int numberlangue) {m_numberLangue=numberlangue;};
	void setModeAndModeNames(QString modename,vector<QString> modenames);
	
	bool findExpAndColorizeSearchString(QString& toaffiche,QRegExp exp);
	int colorizeSearchString(QString &toColorize,int start, int len);


public slots:
	void loadEntry(Entry *entry);
	void loadItem(QListBoxItem *);
	void loadItemNReformat( QListBoxItem *item );
	void loadcurrent(void);
	void goodPosition(void);
	void textHasChanged();
	void modeHasChanged(int i);
	void loadCurrentWithoutUpdatingSearchField();

public :

	Page( CentralWidget* cwidg, Dico* mydico, QWidget * parent=0, const char * name=0, WFlags f=0, bool allowLines=TRUE );

	CentralWidget* centralWidget() {return m_cwidg;};

	MyBrowser* browser(void) {return m_browser;};
	void menage(void) {qlbm->menage();};
	void insertefin(Entry* newone) {qlbm->insertefin(newone);};
	ListBoxManager* manager(void) {return qlbm;};
	QListBox* listBox(void) {return qlb;};
	void loadCurrentPub(void);
	void loadZero(void);
	void loadNumber(int i);
	void setCurrentItem( int i );

	int nbWords();

	int indexof(QString word);

	void searchExp(QRegExp exp,bool casesens,bool inentrynames);

	QString fileName() {return m_fileName;};
	QString title() {return m_title;};
	Page* Voisin() {return voisin;};
	int numberLangue() {return m_numberLangue;};

	void setVoisin(Page* vois) {voisin=vois;};
	void setProperties(QString title,QString fileName, int numberlangue,QString modename,vector<QString> modenames)
	{
		setTitle(title);setFileName(fileName);setNumberLangue(numberlangue);
		setModeAndModeNames(modename, modenames);
	};

	void tengwarRender(QString toaffiche);
	int currentMode() {return m_mode;};

	SearchField* searchField() {return search;};
	QTabWidget* tabWidget() {return m_tbw;};

	
};


#endif
