/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#include "searchinterface.h"
#include "centralwidget.h"
#include "preferences.h"

#include <qlayout.h>
#include <qbuttongroup.h>

class EnterButton : public QPushButton
{
	SearchInterface *m_si;

public :
	EnterButton( SearchInterface *si, QWidget *groupbox)
		: QPushButton( "Search", groupbox ), m_si( si )
	{}

	void keyPressEvent ( QKeyEvent *e )
	{
		if ( (e->key() == Key_Return) || (e->key() == Key_Enter) )
		{
			m_si->roger();
		}
	}
};



SearchInterface::SearchInterface( MainWindow *mwin, const char *name, WFlags)
	: QDialog( mwin, name)
{

	father = mwin;

	QBoxLayout *l = new QVBoxLayout( this );

	groupbox = new QVGroupBox( this );
	groupbox->setMargin( 2 );
	groupbox->setTitle( "Enter a regular expression..." );

	linedit = new QLineEdit( groupbox );

	QString unifontname = DFConfig::instance()->getString( "fonts", "unicode", "Lucida Sans Unicode" );
	linedit->setFont( QFont( unifontname, 8 ));

	QButtonGroup* bg = new QButtonGroup( 2, Horizontal, "Where ?", groupbox );
	bg->setExclusive( true );
	bg->setMargin( 0 );
	searchinentrynames  = new QRadioButton( "In entry names", bg );
	searchindescitpions = new QRadioButton( "In entry descriptions", bg );
	bg->insert( searchinentrynames );
	bg->insert( searchindescitpions );
	searchinentrynames->setChecked( true );

	QHBox *hb = new QHBox( groupbox );

	casesensitive = new QCheckBox( "Case Sensitive", hb );
	EnterButton *benter = new EnterButton( this, hb );
	
	l->addWidget( groupbox );

	benter->setFixedWidth( 100 );
//	groupbox->setFixedSize(290,125);
	setCaption( "Search..." );
//	setFixedSize(300,130);

	connect(benter, SIGNAL(pressed()), this, SLOT( rogerslot() ) );
	connect(linedit, SIGNAL(returnPressed()), this, SLOT( rogerslot() ) );
}

void SearchInterface::popUp()
{
	linedit->setFocus();
	this->show();
//	showMaximized();
}


void SearchInterface::rogerslot( void )
{
	roger();
}

void SearchInterface::roger( void )
{
	father->centralWidget()->currentPage()->searchExp( QRegExp( linedit->text() ),
													   casesensitive->isChecked(),
													   searchinentrynames->isChecked() );
}

/* EOF */
