/*
 * Copyright (C) Dragon Flame, Benjamin Babut and Didier Willis
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation, 
 * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#ifdef UNICODE
/*
 * Ce module n'exploite pas l'Unicode, quand bien mme on 
 * chercherait  lui imposer.
 */
#undef UNICODE
#endif


#ifdef WIN32
#include <windows.h>
#include <wininet.h>
#include <stdio.h>
#include "zlib.h"
#pragma comment(lib, "wininet.lib")
#pragma comment(lib, "zlib.lib")
#else
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#endif



#ifdef CONSOLE
#define TXTDEBUG( a, b ) printf( a, b )
#else
#define TXTDEBUG( a, b )
#endif


/* ------------------------ */
/*
 * httpRetrieveStamp
 *    Effectue une requete HEAD et formatte une etiquette "GMT time [size]"
 *    La taille du fichier est aussi retournee (0 si erreur)
 */

#ifdef WIN32
DWORD httpRetrieveStamp( HINTERNET hConnect,
				         const char *lpcURL, 
					     char *lpStamp )
{
	char  *lpHeader = 0;
	DWORD dwSize;
	DWORD dwSizeHeader;
	BOOL  fOk;
		
	/* -- Requete HEAD */

	HINTERNET hRequest = ::HttpOpenRequest( 
		hConnect, 
		"HEAD", 
		lpcURL,
		0, 0, 0, 
		INTERNET_FLAG_RELOAD, 0 );
	fOk = ::HttpSendRequest( hRequest, 0, 0, 0, 0 );
	if (!fOk) 
	{
		/* Echec la requete (URL incorrecte, etc.) */
		::InternetCloseHandle( hRequest );
		return( 0 );
	}

	/* -- Recuperation Content-Length et Last-Modified */

	::HttpQueryInfo( hRequest, HTTP_QUERY_RAW_HEADERS_CRLF, 0, &dwSizeHeader, 0 );
	lpHeader = new char [dwSizeHeader +1 ];

	dwSize = dwSizeHeader;
	fOk = ::HttpQueryInfo( hRequest, HTTP_QUERY_STATUS_CODE, lpHeader, &dwSize, 0 );
	lpHeader[dwSize] = 0;
	if ( (!fOk) || (strcmp( lpHeader, "200" )) )
	{
		/* Echec (page indisponible, etc.) */
		delete [] lpHeader;
		::InternetCloseHandle( hRequest );
		return( 0 );
	}

	dwSize = dwSizeHeader;
	fOk = ::HttpQueryInfo( hRequest, HTTP_QUERY_LAST_MODIFIED, lpHeader, &dwSize, 0 );
	if (!fOk) 
	{
		/* Absence du champ Last-Modified
		 * Ce cas ne devrait pas se produire
		 */
		delete [] lpHeader;
		::InternetCloseHandle( hRequest );
		return( 0 );
	}
	lpHeader[dwSize] = 0;
	strcpy( lpStamp, lpHeader );

	dwSize = dwSizeHeader;
	fOk = ::HttpQueryInfo( hRequest, HTTP_QUERY_CONTENT_LENGTH, lpHeader, &dwSize, 0);
	if (!fOk)
	{
		/* Absence du champ Content-Length
		 * Ce cas ne devrait pas se produire
		 */
		delete [] lpHeader;
		::InternetCloseHandle( hRequest );
		return( 0 );
	}
	lpHeader[dwSize] = 0;
	sprintf( lpStamp, "%s [%s]", lpStamp, lpHeader );
	dwSize = atoi( lpHeader );

	delete [] lpHeader;
	::InternetCloseHandle( hRequest );
	return( dwSize );
}

/* ------------------------ */

/*
 * httpDownloadFile
 *    Charge un fichier et retourne 'true' en cas
 *    de succes.
 */

bool httpDownloadFile( HINTERNET hConnect,
					   const char *lpcURL,
					   const char *lpcLocalName,
					   const DWORD dwSize )
{
	char  *lpBody    = 0;
	DWORD dwDataSize = 0;
	DWORD dwRealSize = 0;
	BOOL  fOk;

	/* -- Requete GET */

	HINTERNET hRequest = ::HttpOpenRequest( 
		hConnect, 
		"GET", 
		lpcURL,
		0, 0, 0, 
		INTERNET_FLAG_RELOAD, 0 );
	fOk = ::HttpSendRequest( hRequest, 0, 0, 0, 0 );
	if (!fOk) 
	{
		/* Echec la requete (URL incorrecte, etc.) */
		::InternetCloseHandle( hRequest );
		return( false );
	}

	/* -- Recuperation des donnees */

	lpBody = new char [dwSize + 1];
	do 
	{
		fOk = ::InternetReadFile( hRequest, lpBody + dwDataSize, dwSize, &dwDataSize);
		if (!fOk) 
		{
			break;
		}
		dwRealSize += dwDataSize;
	}
	while (dwDataSize != 0);
	::InternetCloseHandle( hRequest );

	/* -- Verification du 'magic number' -- */
	if ((dwRealSize > 2) &&
		((unsigned char)lpBody[0] == 0x1F) && ((unsigned char)lpBody[1] == 0x8B))
	{
		/* -- Creation d'un fichier */

		FILE *fd = fopen( lpcLocalName, "wb" );
		if (fd != 0)
		{
			fwrite( lpBody, 1, dwRealSize, fd );
			fclose( fd );
		}
	}
	else
	{
		/* WARNING
		 *   On est oblig de faire a, c'est  dire de jeter
		 *   si le fichier n'est pas un GZIP (signature 1F8B),
		 *   car il arrive que les sites webs retournent OK comme
		 *   status plutot qu'une erreur 404 ou autre, avec en
		 *   contenu une simple page web d'erreur... auquel cas
		 *   on craserait les fichiers avec ce contenu...
		 */
		delete [] lpBody;
		return( false );
	}

	delete [] lpBody;
	return( true );
}

/* ------------------ */

/*
 * zlibUncompress
 *    Decompresse un fichier (ou le laisse
 *    intact s'il est non compresse)
 */

#define MAXFILE_SZ 2000000 /* 2 Mo MAXIMUM */

void zlibUncompress( const char *lpcLocalName )
{
	DWORD dwBufferSize;
	Bytef *lpBuffer = new Bytef[MAXFILE_SZ];
	
	/* -- Ouverture du fichier, avec decompression
	 *    le cas echeant.
	 */

	gzFile fdz = gzopen( lpcLocalName, "rb" );
	if (fdz != 0)
	{
		dwBufferSize = gzread( fdz, lpBuffer, MAXFILE_SZ );
		gzclose( fdz );
	}

	/* -- Reecriture du fichier */

	FILE *fd = fopen( lpcLocalName, "wb" );
	if (fd != 0)
	{
		fwrite( lpBuffer, 1, dwBufferSize, fd );
		fclose( fd );
	}
	delete [] lpBuffer;
}


/* ------------------ */

/*
 * httpDownloadIfModified
 *    Compare le fichier sur le web a son etiquette
 *    locale, et le ramene integralement si une version
 *    differente est presente.
 *    Retourne 0 si le fichier est deja a jour, 1 sinon.
 */

#define STAMP_SZ 100

int httpDownloadIfModified( HINTERNET hConnect,
					        const char *lpcURL,
						    const char *lpcLocalName )
{
	char  lpStampFile[MAX_PATH];
	char  lpNewStamp[STAMP_SZ];
	char  lpTmpStamp[STAMP_SZ];
	char  *lpOldStamp = 0;
	DWORD dwSize;
	int   code = 0;
	
	dwSize = httpRetrieveStamp( hConnect, lpcURL, lpNewStamp );
	if (dwSize != 0)
	{
		FILE *fds;

		sprintf( lpStampFile, "%s.dat", lpcLocalName );

		fds = fopen( lpStampFile, "r" );
		if (fds != 0)
		{
			lpOldStamp = fgets( lpTmpStamp, STAMP_SZ, fds );
			fclose( fds );

			if ( (lpOldStamp == 0) || (strcmp( lpOldStamp, lpNewStamp )) )
			{
				bool bOk;
				TXTDEBUG( "Newer version of file %s is retrieved\n", lpcLocalName );

				bOk = httpDownloadFile( hConnect, lpcURL, lpcLocalName, dwSize );
				if (bOk)
				{
					/* -- Decompression */
					zlibUncompress( lpcLocalName );

					/* -- Sauvegarde nouvelle etiquette */
					fds = fopen( lpStampFile, "w" );
					if (fds != 0)
					{
						fputs( lpNewStamp, fds );
						fclose( fds );
					}
					return( 1 );
				}
				TXTDEBUG( "Site file %s is not valid\n", lpcLocalName );
				return( 0 );
			}
			else
			{
				TXTDEBUG( "File %s is up-to-date\n", lpcLocalName );
				return( 0 );
			}
		}
		else
		{
			TXTDEBUG( "File %s ignored (no stamp)\n", lpcLocalName );
			return( 0 );
		}
	}
	else
	{
		TXTDEBUG( "File %s ignored (no image)\n", lpcLocalName );
	}
    return( 0 );
}

/* ------------------ */

/*
 * httpSynchronize
 *    Met a jour tous les fichiers du repertoire indique,
 *    s'il possede une etiquette associee (.dat).
 *    Retourne le nombre de fichiers mis  jour.
 */

#define LIST_SZ 50

#endif

#ifdef WIN32
#else
#define DWORD int
#endif

#ifdef WIN32 //TODO temporaire (pas utilis pour l'instant
int httpSynchronize( const char *lpcSite,
				     const char *lpcURLBase,
					 const char *lpcDirBase )
#else
int httpSynchronize( const char *,
				     const char *,
					 const char *)
#endif
{
	DWORD count = 0;

	/* -- Session internet */

#ifdef WIN32
	HINTERNET hSession = ::InternetOpen( 
		"Dragon Flame Agent", 
		INTERNET_OPEN_TYPE_DIRECT, 
		0, 0, 0 );
	if (!hSession) 
	{
		return -1;
	}

	/* -- Connection au site */

	HINTERNET hConnect = ::InternetConnect( 
			hSession,
			lpcSite,
			INTERNET_DEFAULT_HTTP_PORT, 
			0, 0, 
			INTERNET_SERVICE_HTTP, 
			0, 0 );
  	if (!hConnect) 
	{ 
		return -1;
	}

	/* -- Mise a jour de tous les fichiers dans le repertoire */
	{
		WIN32_FIND_DATA strcFindData;
		HANDLE          hFind;
		char szFileName[MAX_PATH];
		char szURLName[MAX_PATH];

		sprintf( szFileName, "%s/*", lpcDirBase );
		hFind = FindFirstFile( szFileName, &strcFindData );
    
		if (hFind != INVALID_HANDLE_VALUE) 
		{    
			do
			{
				if ( ( strcFindData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY )
				     || (strstr( strcFindData.cFileName, ".dat" ) ) )
				{
					continue;
				}

				sprintf( szFileName, "%s/%s", lpcDirBase, strcFindData.cFileName );
				sprintf( szURLName,  "%s/%s", lpcURLBase, strcFindData.cFileName );

				count += httpDownloadIfModified( hConnect, szURLName, szFileName ); 
			} 
			while( FindNextFile( hFind, &strcFindData ) );
    
			FindClose( hFind );
		}
	}

	TXTDEBUG( "Files updated: %ld\n", count );

	::InternetCloseHandle(hConnect);
	::InternetCloseHandle(hSession);
	
	#endif
	return( count );
}

/* EOF */
